"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCompiler = void 0;
const defaultOperatorsMap_1 = require("./defaultOperatorsMap");
function createCompiler(operatorsMap = defaultOperatorsMap_1.defaultOperationsMap) {
    const compiler = template => {
        switch (typeof template) {
            case 'number':
            case 'string':
            case 'boolean':
                return () => template;
            case 'object':
                if (template === null)
                    return () => null;
                if (Array.isArray(template)) {
                    return props => template.map(item => compiler(item)(props));
                }
                for (let i = 0; i < operatorNames.length; i++) {
                    const name = operatorNames[i];
                    let parameters = template[name];
                    if (parameters !== undefined) {
                        if (Object.keys(template).length !== 1) {
                            throw new Error('only one key is allowed');
                        }
                        const operator = operators[i];
                        return operator(parameters);
                    }
                }
                return props => {
                    const res = {};
                    Object.keys(template).forEach(key => {
                        res[key] = compiler(template[key])(props);
                    });
                    return res;
                };
            default:
                return () => undefined;
        }
    };
    const operatorNames = Object.keys(operatorsMap);
    const operators = operatorNames.map(name => operatorsMap[name](compiler));
    return compiler;
}
exports.createCompiler = createCompiler;
//# sourceMappingURL=createCompiler.js.map