"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultOperationsMap = void 0;
const tools_1 = require("./tools");
/**
 * Extract value from object by path
 * @param target Target object
 * @param path path array
 * @returns value or undefined
 */
const extract = (target, path) => {
    if (path.length === 0) {
        return target;
    }
    else {
        if (!tools_1.isObject(target))
            return undefined;
        const [key, ...subpath] = path;
        const targetField = target === null || target === void 0 ? void 0 : target[key];
        if (targetField) {
            return extract(targetField, subpath);
        }
        else {
            return undefined;
        }
    }
};
exports.defaultOperationsMap = {
    // Extract value from properties
    '@': compiler => value => props => {
        const compiledValue = compiler(value)(props);
        const path = Array.isArray(compiledValue)
            ? compiledValue
            : [compiledValue];
        return extract(props, path.filter((p) => typeof p === 'string'));
    },
    // Avoid processing of value
    '@escape': compiler => value => props => value,
    // logical operators
    '@not': tools_1.unar(x => !x),
    '@and': tools_1.binar(tools_1.maybeNumbers((x, y) => x && y)),
    '@or': tools_1.binar(tools_1.maybeNumbers((x, y) => x || y)),
    // comparation operators
    '@lt': tools_1.binar(tools_1.maybeNumbers((x, y) => x < y)),
    '@le': tools_1.binar(tools_1.maybeNumbers((x, y) => x <= y)),
    '@eq': tools_1.binar(tools_1.maybeNumbers((x, y) => x === y)),
    '@ge': tools_1.binar(tools_1.maybeNumbers((x, y) => x >= y)),
    '@gt': tools_1.binar(tools_1.maybeNumbers((x, y) => x > y)),
    // mathematical operators
    '@add': tools_1.binar(tools_1.maybeNumbers((x, y) => x + y)),
    '@sub': tools_1.binar(tools_1.maybeNumbers((x, y) => x - y)),
    '@mul': tools_1.binar(tools_1.maybeNumbers((x, y) => x * y)),
    '@div': tools_1.binar(tools_1.maybeNumbers((x, y) => x / y)),
    //
    '@if': tools_1.triar((opt, yes, no) => (opt ? yes : no)),
    // array
    '@map': compiler => value => {
        if (!Array.isArray(value)) {
            throw new Error();
        }
        if (value.length !== 4) {
            throw new Error();
        }
        const list = compiler(value[0]);
        const map = value[1];
        const valueTemplate = compiler(value[2]);
        const indexTemplate = compiler(value[3]);
        return props => {
            if (typeof props === 'object') {
                const compiledList = list(props);
                const valueKey = valueTemplate(props);
                const valueName = typeof valueKey === 'string' ? valueKey : 'value';
                const indexKey = indexTemplate(props);
                const indexName = typeof indexKey === 'string' ? indexKey : 'index';
                if (tools_1.isArray(compiledList))
                    return compiledList.map((value, index) => {
                        const extended = Object.assign(Object.assign({}, props), { [valueName]: value, [indexName]: index });
                        return compiler(map)(extended);
                    });
            }
        };
    },
};
//# sourceMappingURL=defaultOperatorsMap.js.map